@@ -38,7 +38,7 @@ use tracing_futures::Instrument;
 ///
 /// ```no_run
 /// # async fn foo() -> Result<(), Box<dyn std::error::Error>> {
-/// use ethers::providers::{Middleware, Provider, Http};
+/// use ethers_providers::{Middleware, Provider, Http};
 /// use std::convert::TryFrom;
 ///
 /// let provider = Provider::<Http>::try_from(
@@ -167,7 +167,8 @@ impl<P: JsonRpcClient> Provider<P> {
 }
 
 #[cfg(feature = "celo")]
-#[async_trait]
+#[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
+#[cfg_attr(not(target_arch = "wasm32"), async_trait)]
 impl<P: JsonRpcClient> CeloMiddleware for Provider<P> {
     async fn get_validators_bls_public_keys<T: Into<BlockId> + Send + Sync>(
         &self,
@@ -179,7 +180,8 @@ impl<P: JsonRpcClient> CeloMiddleware for Provider<P> {
     }
 }
 
-#[async_trait]
+#[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
+#[cfg_attr(not(target_arch = "wasm32"), async_trait)]
 impl<P: JsonRpcClient> Middleware for Provider<P> {
     type Error = ProviderError;
     type Provider = P;
@@ -227,6 +229,47 @@ impl<P: JsonRpcClient> Middleware for Provider<P> {
         self.get_block_gen(block_hash_or_number.into(), true).await
     }
 
+    /// Gets the block uncle count at `block_hash_or_number`
+    async fn get_uncle_count<T: Into<BlockId> + Send + Sync>(
+        &self,
+        block_hash_or_number: T,
+    ) -> Result<U256, Self::Error> {
+        let id = block_hash_or_number.into();
+        Ok(match id {
+            BlockId::Hash(hash) => {
+                let hash = utils::serialize(&hash);
+                self.request("eth_getUncleCountByBlockHash", [hash]).await?
+            }
+            BlockId::Number(num) => {
+                let num = utils::serialize(&num);
+                self.request("eth_getUncleCountByBlockNumber", [num])
+                    .await?
+            }
+        })
+    }
+
+    /// Gets the block uncle at `block_hash_or_number` and `idx`
+    async fn get_uncle<T: Into<BlockId> + Send + Sync>(
+        &self,
+        block_hash_or_number: T,
+        idx: U64,
+    ) -> Result<Option<Block<H256>>, ProviderError> {
+        let blk_id = block_hash_or_number.into();
+        let idx = utils::serialize(&idx);
+        Ok(match blk_id {
+            BlockId::Hash(hash) => {
+                let hash = utils::serialize(&hash);
+                self.request("eth_getUncleByBlockHashAndIndex", [hash, idx])
+                    .await?
+            }
+            BlockId::Number(num) => {
+                let num = utils::serialize(&num);
+                self.request("eth_getUncleByBlockNumberAndIndex", [num, idx])
+                    .await?
+            }
+        })
+    }
+
     /// Gets the transaction with `transaction_hash`
     async fn get_transaction<T: Send + Sync + Into<TxHash>>(
         &self,
@@ -261,6 +304,37 @@ impl<P: JsonRpcClient> Middleware for Provider<P> {
         self.request("eth_gasPrice", ()).await
     }
 
+    /// Gets a heuristic recommendation of max fee per gas and max priority fee per gas for
+    /// EIP-1559 compatible transactions.
+    async fn estimate_eip1559_fees(
+        &self,
+        estimator: Option<fn(U256, Vec<Vec<U256>>) -> (U256, U256)>,
+    ) -> Result<(U256, U256), Self::Error> {
+        let base_fee_per_gas = self
+            .get_block(BlockNumber::Latest)
+            .await?
+            .ok_or_else(|| ProviderError::CustomError("Latest block not found".into()))?
+            .base_fee_per_gas
+            .ok_or_else(|| ProviderError::CustomError("EIP-1559 not activated".into()))?;
+
+        let fee_history = self
+            .fee_history(
+                utils::EIP1559_FEE_ESTIMATION_PAST_BLOCKS,
+                BlockNumber::Latest,
+                &[utils::EIP1559_FEE_ESTIMATION_REWARD_PERCENTILE],
+            )
+            .await?;
+
+        // use the provided fee estimator function, or fallback to the default implementation.
+        let (max_fee_per_gas, max_priority_fee_per_gas) = if let Some(es) = estimator {
+            es(base_fee_per_gas, fee_history.reward)
+        } else {
+            utils::eip1559_default_estimator(base_fee_per_gas, fee_history.reward)
+        };
+
+        Ok((max_fee_per_gas, max_priority_fee_per_gas))
+    }
+
     /// Gets the accounts on the node
     async fn get_accounts(&self) -> Result<Vec<Address>, ProviderError> {
         self.request("eth_accounts", ()).await
@@ -697,20 +771,37 @@ impl<P: JsonRpcClient> Middleware for Provider<P> {
         self.subscribe([logs, filter]).await
     }
 
-    async fn fee_history(
+    async fn fee_history<T: Into<U256> + serde::Serialize + Send + Sync>(
         &self,
-        block_count: u64,
+        block_count: T,
         last_block: BlockNumber,
         reward_percentiles: &[f64],
     ) -> Result<FeeHistory, Self::Error> {
-        let block_count = utils::serialize(&block_count);
         let last_block = utils::serialize(&last_block);
         let reward_percentiles = utils::serialize(&reward_percentiles);
+
+        // The blockCount param is expected to be an unsigned integer up to geth v1.10.6.
+        // Geth v1.10.7 onwards, this has been updated to a hex encoded form. Failure to
+        // decode the param from client side would fallback to the old API spec.
         self.request(
             "eth_feeHistory",
-            [block_count, last_block, reward_percentiles],
+            [
+                utils::serialize(&block_count),
+                last_block.clone(),
+                reward_percentiles.clone(),
+            ],
         )
         .await
+        .or(self
+            .request(
+                "eth_feeHistory",
+                [
+                    utils::serialize(&block_count.into().as_u64()),
+                    last_block,
+                    reward_percentiles,
+                ],
+            )
+            .await)
     }
 }
 
@@ -781,14 +872,23 @@ impl<P: JsonRpcClient> Provider<P> {
 #[cfg(feature = "ws")]
 impl Provider<crate::Ws> {
     /// Direct connection to a websocket endpoint
+    #[cfg(not(target_arch = "wasm32"))]
     pub async fn connect(
         url: impl tokio_tungstenite::tungstenite::client::IntoClientRequest + Unpin,
     ) -> Result<Self, ProviderError> {
         let ws = crate::Ws::connect(url).await?;
         Ok(Self::new(ws))
     }
+
+    /// Direct connection to a websocket endpoint
+    #[cfg(target_arch = "wasm32")]
+    pub async fn connect(url: &str) -> Result<Self, ProviderError> {
+        let ws = crate::Ws::connect(url).await?;
+        Ok(Self::new(ws))
+    }
 }
 
+#[cfg(not(target_arch = "wasm32"))]
 #[cfg(feature = "ipc")]
 impl Provider<crate::Ipc> {
     /// Direct connection to an IPC socket.
@@ -805,7 +905,8 @@ impl Provider<MockProvider> {
     ///
     /// ```
     /// # async fn foo() -> Result<(), Box<dyn std::error::Error>> {
-    /// use ethers::{types::U64, providers::{Middleware, Provider}};
+    /// use ethers_core::types::U64;
+    /// use ethers_providers::{Middleware, Provider};
     /// // Instantiate the provider
     /// let (provider, mock) = Provider::mocked();
     /// // Push the mock response
@@ -854,8 +955,13 @@ impl TryFrom<String> for Provider<HttpProvider> {
 }
 
 #[cfg(test)]
-mod ens_tests {
+#[cfg(not(target_arch = "wasm32"))]
+mod tests {
     use super::*;
+    use crate::Http;
+    use ethers_core::types::{TransactionRequest, H256};
+    use ethers_core::utils::Geth;
+    use futures_util::StreamExt;
 
     const INFURA: &str = "https://mainnet.infura.io/v3/c60b0bb42f8a4c6481ecd229eddaca27";
 
@@ -900,15 +1006,6 @@ mod ens_tests {
             .await
             .unwrap_err();
     }
-}
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use crate::Http;
-    use ethers_core::types::{TransactionRequest, H256};
-    use ethers_core::utils::Geth;
-    use futures_util::StreamExt;
 
     #[tokio::test]
     #[cfg_attr(feature = "celo", ignore)]
@@ -1051,7 +1148,7 @@ mod tests {
         .unwrap();
 
         let history = provider
-            .fee_history(10, BlockNumber::Latest, &[10.0, 40.0])
+            .fee_history(10u64, BlockNumber::Latest, &[10.0, 40.0])
             .await
             .unwrap();
         dbg!(&history);
